@@ -6,16 +6,20 @@ import gtk
 import hildon
 from os import listdir, path, system
 from ConfigParser import SafeConfigParser
+import dbus
+from dbus.mainloop.glib import DBusGMainLoop
 
-version = "1.6-6"
+version = "2.0-0"
 trackingEnabled = True
 stopTracking = True
 stopWhenIdle = False
 stopAfterMinutes = "5 minutes"
 confirmDisconnect = True
-showRestartButton = False
+showRestartButton = True
+showSettingsButton = True
 showConnected = True
 connectedIcon = "Symbol"
+wifiPMIcon = True
 skipSystemCheck = False
 autostart = False
 keepWifiPMOff = False
@@ -25,7 +29,7 @@ wifiPMOnSec = "4 seconds"
 configReverseOrder = False
 configShowWifiHint = True
 
-class Main(object):
+class Window(object):
     def __init__(self):
         global trackingEnabled
         global stopTracking
@@ -33,8 +37,10 @@ class Main(object):
         global stopAfterMinutes
         global showConnected
         global connectedIcon
+        global wifiPMIcon
         global confirmDisconnect
         global showRestartButton
+        global showSettingsButton
         global skipSystemCheck
         global autostart
         global keepWifiPMOff
@@ -67,8 +73,10 @@ class Main(object):
             self.parser.set('options', 'stopAfterMinutes', str(stopAfterMinutes))
             self.parser.set('options', 'confirmDisconnect', str(confirmDisconnect))
             self.parser.set('options', 'showRestartButton', str(showRestartButton))
+            self.parser.set('options', 'showSettingsButton', str(showSettingsButton))
             self.parser.set('options', 'showConnected', str(showConnected))
             self.parser.set('options', 'connectedIcon', str(connectedIcon))
+            self.parser.set('options', 'wifiPMIcon', str(wifiPMIcon))
             self.parser.set('options', 'skipSystemCheck', str(skipSystemCheck))
             self.parser.set('options', 'configReverseOrder', str(configReverseOrder))
             self.parser.set('options', 'configShowWifiHint', str(configShowWifiHint))
@@ -101,6 +109,10 @@ class Main(object):
                     showRestartButton = self.parser.getboolean('options', 'showRestartButton')
                 else:
                     self.parser.set('options', 'showRestartButton', str(showRestartButton))
+                if self.parser.has_option('options', 'showSettingsButton'):
+                    showSettingsButton = self.parser.getboolean('options', 'showSettingsButton')
+                else:
+                    self.parser.set('options', 'showSettingsButton', str(showSettingsButton))
                 if self.parser.has_option('options', 'showConnected'):
                     showConnected = self.parser.getboolean('options', 'showConnected')
                 else:
@@ -109,6 +121,10 @@ class Main(object):
                     connectedIcon = self.parser.get('options', 'connectedIcon')
                 else:
                     self.parser.set('options', 'connectedIcon', str(connectedIcon))
+                if self.parser.has_option('options', 'wifiPMIcon'):
+                    wifiPMIcon = self.parser.getboolean('options', 'wifiPMIcon')
+                else:
+                    self.parser.set('options', 'wifiPMIcon', str(wifiPMIcon))
                 if self.parser.has_option('options', 'skipSystemCheck'):
                     skipSystemCheck = self.parser.getboolean('options', 'skipSystemCheck')
                 else:
@@ -186,8 +202,23 @@ class Main(object):
         self.pmOffList = ["1 second", "2 seconds", "3 seconds"]
         self.pmOnList = ["3 seconds", "4 seconds", "5 seconds"]
         
-        self.main()
-    
+        DBusGMainLoop(set_as_default=True)
+        bus = dbus.SessionBus()
+        bus.add_signal_receiver(self.dbus, path='/', dbus_interface='org.maemo.oswoptions', signal_name='changed2')
+        
+    def dbus(self, param, val):
+        if param == "keepWifiPMOff":
+            self.buttonKeepWifiPMOff_dbus()
+ 
+        if param == "toggleWifiPM":
+            self.buttonToggleWifiPM_dbus()
+ 
+        if param == "wifiPMOffSec":
+            self.buttonWifiPMOffSec_dbus(val)
+ 
+        if param == "wifiPMOnSec":
+            self.buttonWifiPMOnSec_dbus(val)
+ 
     def main(self):
         self.window = hildon.Window()
         self.window.set_title("OpenSSH Status Widget")
@@ -195,17 +226,18 @@ class Main(object):
         
         self.labelWidget = gtk.Label("Widget options:")
         self.labelServer = gtk.Label("Server options:")
+        self.labelWifiPM = gtk.Label("Wifi PM options:")
     
-        self.buttonTrackingEnabled = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonTrackingEnabled = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonTrackingEnabled.set_label("Enable tracking")
     
-        self.buttonStopTracking = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonStopTracking = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonStopTracking.set_label("Stop tracking when server is off")
     
-        self.buttonStopWhenIdle = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonStopWhenIdle = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonStopWhenIdle.set_label("Stop server when not connected")
     
-        self.buttonStopAfterMinutes = hildon.PickerButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
+        self.buttonStopAfterMinutes = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
         self.buttonStopAfterMinutes.set_title("Wait:")
         self.buttonStopAfterMinutes.set_alignment(0.09, 0.5, 0, 0)
         self.buttonStopAfterMinutes.set_selector(self.selectTimeout())
@@ -214,17 +246,21 @@ class Main(object):
         except:
             self.buttonStopAfterMinutes.set_active(1)
     
-        self.buttonConfirm = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
-        self.buttonConfirm.set_label("Confirm when disconnecting")
+        self.buttonConfirm = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonConfirm.set_label("Show dialog on long tap (do not disconnect)")
         
-        self.buttonShowRestart = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
-        self.buttonShowRestart.set_label("Show Restart button")
+        self.buttonShowRestart = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonShowRestart.set_label("Show restart button")
         self.buttonShowRestart.set_alignment(0.08, 0.5)
     
-        self.buttonShowConnected = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonShowSettings = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonShowSettings.set_label("Show settings button")
+        self.buttonShowSettings.set_alignment(0.083, 0.5)
+    
+        self.buttonShowConnected = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonShowConnected.set_label("Show status 'connected' in status area")
     
-        self.buttonConnectedIcon = hildon.PickerButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
+        self.buttonConnectedIcon = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
         self.buttonConnectedIcon.set_title("Indicator:")
         self.buttonConnectedIcon.set_alignment(0.09, 0.5, 0, 0)
         self.buttonConnectedIcon.set_selector(self.selectIndicator())
@@ -232,18 +268,21 @@ class Main(object):
             self.buttonConnectedIcon.set_active(self.indicatorList.index(connectedIcon.strip('_')))
         except:
             self.buttonConnectedIcon.set_active(0)
+        
+        self.buttonWifiPMIcon = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonWifiPMIcon.set_label("Show wifi power management button")
     
-        self.buttonAutostart = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonAutostart = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonAutostart.set_label("Run server at system startup")
                 
-        self.buttonKeepWifiPMOff = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonKeepWifiPMOff = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonKeepWifiPMOff.set_label("Keep wifi power management off when connected")
     
-        self.buttonToggleWifiPM = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonToggleWifiPM = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonToggleWifiPM.set_label("Toggle mode (less power consuming)")
         self.buttonToggleWifiPM.set_alignment(0.12, 0.5)
     
-        self.buttonWifiPMOffSec = hildon.PickerButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
+        self.buttonWifiPMOffSec = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
         self.buttonWifiPMOffSec.set_title("Wifi PM off:")
         self.buttonWifiPMOffSec.set_alignment(0.17, 0.5, 0, 0)
         self.buttonWifiPMOffSec.set_selector(self.selectWifiPMOffSec())
@@ -252,7 +291,7 @@ class Main(object):
         except:
             self.buttonWifiPMOffSec.set_active(1)
     
-        self.buttonWifiPMOnSec = hildon.PickerButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
+        self.buttonWifiPMOnSec = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
         self.buttonWifiPMOnSec.set_title("Wifi PM on:")
         self.buttonWifiPMOnSec.set_alignment(0.17, 0.5, 0, 0)
         self.buttonWifiPMOnSec.set_selector(self.selectWifiPMOnSec())
@@ -267,11 +306,13 @@ class Main(object):
         self.vbox.set_border_width(0)
         
         if configReverseOrder:
+            self.pack_startWifiPMButtons()
             self.pack_startServerButtons()
             self.pack_startWidgetButtons()
         else:
             self.pack_startWidgetButtons()
             self.pack_startServerButtons()
+            self.pack_startWifiPMButtons()
         
         areaScrolling = hildon.PannableArea()
         areaScrolling.add_with_viewport(self.vbox)
@@ -296,15 +337,22 @@ class Main(object):
         if confirmDisconnect:
             self.buttonConfirm.set_active(True)
             self.buttonShowRestart.set_sensitive(True)
+            self.buttonShowSettings.set_sensitive(True)
         else:
             self.buttonConfirm.set_active(False)
             self.buttonShowRestart.set_sensitive(False)
+            self.buttonShowSettings.set_sensitive(False)
 
         if showRestartButton:
             self.buttonShowRestart.set_active(True)
         else:
             self.buttonShowRestart.set_active(False)
 
+        if showSettingsButton:
+            self.buttonShowSettings.set_active(True)
+        else:
+            self.buttonShowSettings.set_active(False)
+
         if showConnected:
             self.buttonShowConnected.set_active(True)
             self.buttonConnectedIcon.set_sensitive(True)
@@ -312,6 +360,11 @@ class Main(object):
             self.buttonShowConnected.set_active(False)
             self.buttonConnectedIcon.set_sensitive(False)
             
+        if wifiPMIcon:
+            self.buttonWifiPMIcon.set_active(True)
+        else:
+            self.buttonWifiPMIcon.set_active(False)
+
         if trackingEnabled:
             self.buttonTrackingEnabled.set_active(True)
         else:
@@ -321,6 +374,7 @@ class Main(object):
             self.buttonStopAfterMinutes.set_sensitive(False)
             self.buttonConfirm.set_sensitive(False)
             self.buttonShowRestart.set_sensitive(True)
+            self.buttonShowSettings.set_sensitive(True)
             self.buttonShowConnected.set_sensitive(False)
             self.buttonConnectedIcon.set_sensitive(False)
         
@@ -351,13 +405,15 @@ class Main(object):
         self.buttonStopAfterMinutes.connect("value-changed", self.buttonStopAfterMinutes_changed)
         self.buttonConfirm.connect("clicked", self.buttonConfirm_clicked)
         self.buttonShowRestart.connect("clicked", self.buttonShowRestart_clicked)
+        self.buttonShowSettings.connect("clicked", self.buttonShowSettings_clicked)
         self.buttonShowConnected.connect("clicked", self.buttonShowConnected_clicked)
         self.buttonConnectedIcon.connect("value-changed", self.buttonConnectedIcon_changed)
+        self.buttonWifiPMIcon.connect("clicked", self.buttonWifiPMIcon_clicked)
         self.buttonAutostart.connect("clicked", self.buttonAutostart_clicked)
-        self.buttonKeepWifiPMOff.connect("clicked", self.buttonKeepWifiPMOff_clicked)
-        self.buttonToggleWifiPM.connect("clicked", self.buttonToggleWifiPM_clicked)
-        self.buttonWifiPMOffSec.connect("value-changed", self.buttonWifiPMOffSec_changed)
-        self.buttonWifiPMOnSec.connect("value-changed", self.buttonWifiPMOnSec_changed)
+        self.buttonKeepWifiPMOff_id = self.buttonKeepWifiPMOff.connect("clicked", self.buttonKeepWifiPMOff_clicked)
+        self.buttonToggleWifiPM_id = self.buttonToggleWifiPM.connect("clicked", self.buttonToggleWifiPM_clicked)
+        self.buttonWifiPMOffSec_id = self.buttonWifiPMOffSec.connect("value-changed", self.buttonWifiPMOffSec_changed)
+        self.buttonWifiPMOnSec_id = self.buttonWifiPMOnSec.connect("value-changed", self.buttonWifiPMOnSec_changed)
         
         menu = self.create_menu()
         self.window.set_app_menu(menu)
@@ -372,14 +428,19 @@ class Main(object):
         self.vbox.pack_start(self.buttonStopTracking, False, False, 0)
         self.vbox.pack_start(self.buttonStopWhenIdle, False, False, 0)
         self.vbox.pack_start(self.buttonStopAfterMinutes, False, False, 0)
-        self.vbox.pack_start(self.buttonConfirm, False, False, 0)
-        self.vbox.pack_start(self.buttonShowRestart, False, False, 0)
         self.vbox.pack_start(self.buttonShowConnected, False, False, 0)
         self.vbox.pack_start(self.buttonConnectedIcon, False, False, 0)
+        self.vbox.pack_start(self.buttonConfirm, False, False, 0)
+        self.vbox.pack_start(self.buttonShowRestart, False, False, 0)
+        self.vbox.pack_start(self.buttonShowSettings, False, False, 0)
+        self.vbox.pack_start(self.buttonWifiPMIcon, False, False, 0)
     
     def pack_startServerButtons(self):
         self.vbox.pack_start(self.labelServer, False, False, 8)
         self.vbox.pack_start(self.buttonAutostart, False, False, 0)
+        
+    def pack_startWifiPMButtons(self):
+        self.vbox.pack_start(self.labelWifiPM, False, False, 8)
         self.vbox.pack_start(self.buttonKeepWifiPMOff, False, False, 0)
         self.vbox.pack_start(self.buttonToggleWifiPM, False, False, 0)
         self.vbox.pack_start(self.buttonWifiPMOffSec, False, False, 0)
@@ -391,14 +452,19 @@ class Main(object):
         self.vbox.remove(self.buttonStopTracking)
         self.vbox.remove(self.buttonStopWhenIdle)
         self.vbox.remove(self.buttonStopAfterMinutes)
-        self.vbox.remove(self.buttonConfirm)
-        self.vbox.remove(self.buttonShowRestart)
         self.vbox.remove(self.buttonShowConnected)
         self.vbox.remove(self.buttonConnectedIcon)
+        self.vbox.remove(self.buttonConfirm)
+        self.vbox.remove(self.buttonShowRestart)
+        self.vbox.remove(self.buttonShowSettings)
+        self.vbox.remove(self.buttonWifiPMIcon)
     
     def removeServerButtons(self):
         self.vbox.remove(self.labelServer)
         self.vbox.remove(self.buttonAutostart)
+        
+    def removeWifiPMButtons(self):
+        self.vbox.remove(self.labelWifiPM)
         self.vbox.remove(self.buttonKeepWifiPMOff)
         self.vbox.remove(self.buttonToggleWifiPM)
         self.vbox.remove(self.buttonWifiPMOffSec)
@@ -415,8 +481,10 @@ class Main(object):
             self.buttonConfirm.set_sensitive(True)
             if confirmDisconnect:
                 self.buttonShowRestart.set_sensitive(True)
+                self.buttonShowSettings.set_sensitive(True)
             else:
                 self.buttonShowRestart.set_sensitive(False)
+                self.buttonShowSettings.set_sensitive(False)
             self.buttonShowConnected.set_sensitive(True)
             if showConnected:
                 self.buttonConnectedIcon.set_sensitive(True)
@@ -426,16 +494,19 @@ class Main(object):
             self.buttonStopAfterMinutes.set_sensitive(False)
             self.buttonConfirm.set_sensitive(False)
             self.buttonShowRestart.set_sensitive(True)
+            self.buttonShowSettings.set_sensitive(True)
             self.buttonShowConnected.set_sensitive(False)
             self.buttonConnectedIcon.set_sensitive(False)
         self.parser.set('options', 'trackingEnabled', str(trackingEnabled))
         self.write_config()
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'trackingEnabled' string:'"+str(trackingEnabled)+"'")
         
     def buttonStopTracking_clicked(self, dummy):
         global stopTracking
         stopTracking = not stopTracking
         self.parser.set('options', 'stopTracking', str(stopTracking))
         self.write_config()
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'stopTracking' string:'"+str(stopTracking)+"'")
         
     def buttonStopWhenIdle_clicked(self, dummy):
         global stopWhenIdle
@@ -446,6 +517,7 @@ class Main(object):
             self.buttonStopAfterMinutes.set_sensitive(False)
         self.parser.set('options', 'stopWhenIdle', str(stopWhenIdle))
         self.write_config()
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'stopWhenIdle' string:'"+str(stopWhenIdle)+"'")
         
     def selectTimeout(self):
         selector = hildon.TouchSelector(text = True)
@@ -457,22 +529,38 @@ class Main(object):
         t = self.buttonStopAfterMinutes.get_value()
         self.parser.set('options', 'stopAfterMinutes', t)
         self.write_config()
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'stopAfterMinutes' string:'"+t+"'")
         
     def buttonConfirm_clicked(self, dummy):
         global confirmDisconnect
         confirmDisconnect = not confirmDisconnect
         if confirmDisconnect:
             self.buttonShowRestart.set_sensitive(True)
+            self.buttonShowSettings.set_sensitive(True)
         else:
             self.buttonShowRestart.set_sensitive(False)
+            self.buttonShowSettings.set_sensitive(False)
+            if showSettingsButton:
+                self.printDialog("Settings can be accessed by opening 'OSW Options' from the application menu, category desktop.")
         self.parser.set('options', 'confirmDisconnect', str(confirmDisconnect))
         self.write_config()
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'confirmDisconnect' string:'"+str(confirmDisconnect)+"'")
 
     def buttonShowRestart_clicked(self, dummy):
         global showRestartButton
         showRestartButton = not showRestartButton
         self.parser.set('options', 'showRestartButton', str(showRestartButton))
         self.write_config()
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'showRestartButton' string:'"+str(showRestartButton)+"'")
+
+    def buttonShowSettings_clicked(self, dummy):
+        global showSettingsButton
+        showSettingsButton = not showSettingsButton
+        if not showSettingsButton:
+            self.printDialog("Settings can be accessed by opening 'OSW Options' from the application menu, category desktop.")
+        self.parser.set('options', 'showSettingsButton', str(showSettingsButton))
+        self.write_config()
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'showSettingsButton' string:'"+str(showSettingsButton)+"'")
 
     def buttonShowConnected_clicked(self, dummy):
         global showConnected
@@ -483,6 +571,7 @@ class Main(object):
             self.buttonConnectedIcon.set_sensitive(False)
         self.parser.set('options', 'showConnected', str(showConnected))
         self.write_config()
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'showConnected' string:'"+str(showConnected)+"'")
         
     def selectIndicator(self):
         selector = hildon.TouchSelector(text = True)
@@ -496,6 +585,14 @@ class Main(object):
             style = style + "_"
         self.parser.set('options', 'connectedIcon', style)
         self.write_config()
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'connectedIcon' string:'"+style+"'")
+        
+    def buttonWifiPMIcon_clicked(self, dummy):
+        global wifiPMIcon
+        wifiPMIcon = not wifiPMIcon
+        self.parser.set('options', 'wifiPMIcon', str(wifiPMIcon))
+        self.write_config()
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'wifiPMIcon' string:'"+str(wifiPMIcon)+"'")
         
     def buttonAutostart_clicked(self, dummy):
         global autostart
@@ -524,7 +621,27 @@ class Main(object):
             self.buttonWifiPMOffSec.set_sensitive(False)
             self.buttonWifiPMOnSec.set_sensitive(False)
             system("sudo openssh-status.sh disable_wifipmoff")
-        
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'keepWifiPMOff' string:'"+str(keepWifiPMOff)+"'")
+    
+    def buttonKeepWifiPMOff_dbus(self):
+        self.buttonKeepWifiPMOff.handler_block(self.buttonKeepWifiPMOff_id)
+        global keepWifiPMOff
+        global configShowWifiHint
+        keepWifiPMOff = not keepWifiPMOff
+        if keepWifiPMOff:
+            self.buttonKeepWifiPMOff.set_active(True)
+            self.buttonToggleWifiPM.set_sensitive(True)
+            self.buttonWifiPMOffSec.set_sensitive(True)
+            self.buttonWifiPMOnSec.set_sensitive(True)
+            if configShowWifiHint:
+                configShowWifiHint = False
+        else:
+            self.buttonKeepWifiPMOff.set_active(False)
+            self.buttonToggleWifiPM.set_sensitive(False)
+            self.buttonWifiPMOffSec.set_sensitive(False)
+            self.buttonWifiPMOnSec.set_sensitive(False)
+        self.buttonKeepWifiPMOff.handler_unblock(self.buttonKeepWifiPMOff_id)
+    
     def buttonToggleWifiPM_clicked(self, dummy):
         global toggleWifiPM
         toggleWifiPM = not toggleWifiPM
@@ -532,6 +649,17 @@ class Main(object):
             system("sudo openssh-status.sh enable_togglepm")
         else:
             system("sudo openssh-status.sh disable_togglepm")
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'toggleWifiPM' string:'"+str(toggleWifiPM)+"'")
+        
+    def buttonToggleWifiPM_dbus(self):
+        self.buttonToggleWifiPM.handler_block(self.buttonToggleWifiPM_id)
+        global toggleWifiPM
+        toggleWifiPM = not toggleWifiPM
+        if toggleWifiPM:
+            self.buttonToggleWifiPM.set_active(True)
+        else:
+            self.buttonToggleWifiPM.set_active(False)
+        self.buttonToggleWifiPM.handler_unblock(self.buttonToggleWifiPM_id)
         
     def selectWifiPMOffSec(self):
         selector = hildon.TouchSelector(text = True)
@@ -540,8 +668,17 @@ class Main(object):
         return selector
         
     def buttonWifiPMOffSec_changed(self, dummy):
-        t = self.buttonWifiPMOffSec.get_value().split(' ')[0]
-        system("sudo openssh-status.sh set_pmoffsec "+t)
+        t = self.buttonWifiPMOffSec.get_value()
+        system("sudo openssh-status.sh set_pmoffsec "+t.split(' ')[0])
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'wifiPMOffSec' string:'"+str(t)+"'")
+        
+    def buttonWifiPMOffSec_dbus(self, val):
+        self.buttonWifiPMOffSec.handler_block(self.buttonWifiPMOffSec_id)
+        try:
+            self.buttonWifiPMOffSec.set_active(self.pmOffList.index(val))
+        except:
+            pass
+        self.buttonWifiPMOffSec.handler_unblock(self.buttonWifiPMOffSec_id)
         
     def selectWifiPMOnSec(self):
         selector = hildon.TouchSelector(text = True)
@@ -550,8 +687,17 @@ class Main(object):
         return selector
         
     def buttonWifiPMOnSec_changed(self, dummy):
-        t = self.buttonWifiPMOnSec.get_value().split(' ')[0]
-        system("sudo openssh-status.sh set_pmonsec "+t)
+        t = self.buttonWifiPMOnSec.get_value()
+        system("sudo openssh-status.sh set_pmonsec "+t.split(' ')[0])
+        system("dbus-send --type=signal / org.maemo.oswoptions.changed1 string:'wifiPMOnSec' string:'"+str(t)+"'")
+        
+    def buttonWifiPMOnSec_dbus(self, val):
+        self.buttonWifiPMOnSec.handler_block(self.buttonWifiPMOnSec_id)
+        try:
+            self.buttonWifiPMOnSec.set_active(self.pmOnList.index(val))
+        except:
+            pass
+        self.buttonWifiPMOnSec.handler_unblock(self.buttonWifiPMOnSec_id)
         
     def write_config(self):
         configFile = open('/home/user/.openssh-status-widget', 'r+')
@@ -561,9 +707,6 @@ class Main(object):
     def create_menu(self):
         menu = hildon.AppMenu()
     
-        buttonRestart = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-        buttonRestart.set_label('Restart status menu')
-        buttonRestart.connect("clicked", self.restart_clicked)
         buttonReverse = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
         buttonReverse.set_label('Reverse sections order')
         buttonReverse.connect("clicked", self.reverse_clicked)
@@ -571,7 +714,6 @@ class Main(object):
         buttonAbout.set_label('Info')
         buttonAbout.connect("clicked", self.about_clicked)
     
-        menu.append(buttonRestart)
         menu.append(buttonReverse)
         menu.append(buttonAbout)
     
@@ -579,18 +721,18 @@ class Main(object):
     
         return menu
     
-    def restart_clicked(self, dummy):
-        system("killall hildon-status-menu")
-        
     def reverse_clicked(self, dummy):
         global configReverseOrder
         configReverseOrder = not configReverseOrder
+        self.removeServerButtons()
         if configReverseOrder:
             self.removeWidgetButtons()
+            self.pack_startServerButtons()
             self.pack_startWidgetButtons()
         else:
-            self.removeServerButtons()
+            self.removeWifiPMButtons()
             self.pack_startServerButtons()
+            self.pack_startWifiPMButtons()
         self.parser.set('options', 'configReverseOrder', str(configReverseOrder))
         self.write_config()
     
@@ -617,15 +759,15 @@ class Main(object):
         
     def buttonForum_clicked(self, dummy):
         self.about.destroy()
-        system("dbus-send --session --type=method_call --print-reply --dest=com.nokia.osso_browser \
+        system("dbus-send --session --type=method_call --dest=com.nokia.osso_browser \
         /com/nokia/osso_browser/request com.nokia.osso_browser.open_new_window string:http://talk.maemo.org/showthread.php?t=95429")
             
     def printDialog(self, msg):
-        system("run-standalone.sh dbus-send --print-reply \
+        system("run-standalone.sh dbus-send \
         --type=method_call --dest=org.freedesktop.Notifications \
         /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteDialog \
         string:\""+msg+"\" uint32:0 string:")
 
 if __name__ == "__main__":
-    Main()
-
+    mywindow = Window()
+    mywindow.main()
